/*$
 Copyright (C) 2013-2019 Azel.

 This file is part of AzPainter.

 AzPainter is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 AzPainter is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
$*/

/*******************************
 * スプラインのポイントバッファ
 *******************************/

#ifndef SPLINE_BUF_H
#define SPLINE_BUF_H

void SplineBuf_init();
void SplineBuf_free();

mBool SplineBuf_start();

mBool SplineBuf_addPoint(mPoint *pt,uint8_t pressure);
mBool SplineBuf_deleteLastPoint(mPoint *ptlast);
void SplineBuf_scrollPoint(mPoint *pt);

void SplineBuf_beginGetPoint();
int SplineBuf_getPoint(mPoint *pt);

mBool SplineBuf_initDraw();
mBool SplineBuf_getNextDraw(mDoublePoint *pt,uint8_t *pressure);

#endif
